#ifndef METADATA_H
#define METADATA_H

#include <config.h>

#ifdef HAVE_GUI
#include <gtk/gtk.h>
#else
#include <glib.h>
#endif

#include "bmpx/list.h"
#include <libhrel/types.h>
#include <libhrel/tuple.h>

typedef enum {
    BMP_DATUM_LOCATION,
    BMP_DATUM_TITLE,
    BMP_DATUM_ARTIST,
    BMP_DATUM_ALBUM,
    BMP_DATUM_GENRE,
    BMP_DATUM_COMMENT,

    BMP_DATUM_TIME,
    BMP_DATUM_TRACK,
    BMP_DATUM_RATING,
    BMP_DATUM_DATE,
    BMP_DATUM_MTIME,
    BMP_DATUM_BITRATE,
    BMP_DATUM_SAMPLERATE,
    BMP_DATUM_COUNT,

    BMP_DATUM_NONE
} BmpMetadatumId;

typedef struct _Track Track;
struct _Track
{
  gchar *location;
  gchar *title;
  gchar *artist;
  gchar *album;
  gchar *genre;
  gchar *comment;

  guint  count;
  guint  time;
  guint  track;
  guint  rating;
  guint  date;
  guint  mtime;
  guint  bitrate;
  guint  samplerate;
};

gchar*
bmp_metadata_get_title (BmpMetadatumId tag_id);

gchar*
bmp_metadata_get_id (BmpMetadatumId tag_id);

const gchar*
bmp_metadata_get_id_static (BmpMetadatumId tag_id);

GType
bmp_metadata_get_type (BmpMetadatumId tag_id);

G_BEGIN_DECLS

/* Metadata Cache Functions */
#define BMP_TYPE_METADATA_CACHE		  (bmp_metadata_cache_get_type ())
#define BMP_METADATA_CACHE(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_METADATA_CACHE, BmpMetadataCache))
#define BMP_METADATA_CACHE_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_METADATA_CACHE, BmpMetadataCacheClass))
#define BMP_IS_METADATA_CACHE(obj)  	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_METADATA_CACHE))
#define BMP_IS_METADATA_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_METADATA_CACHE))
#define BMP_METADATA_CACHE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_METADATA_CACHE, BmpMetadataCacheClass))

typedef struct _BmpMetadataCache BmpMetadataCache;
typedef struct _BmpMetadataCacheClass BmpMetadataCacheClass;
typedef struct _BmpMetadataCachePrivate BmpMetadataCachePrivate;

struct _BmpMetadataCache {
	GObject parent;

        BmpMetadataCachePrivate *private;
};

struct _BmpMetadataCacheClass {
	GObjectClass parent;

	void (*file_added)	(BmpMetadataCache * self, gint _idx);
};

extern BmpMetadataCache   *bmp_metadata_cache;

GType
bmp_metadata_cache_get_type (void);

BmpMetadataCache*
bmp_metadata_cache_new (void);

void
bmp_metadata_cache_write (BmpMetadataCache *self);
void
bmp_metadata_cache_read (BmpMetadataCache *self);

HTuple*
bmp_metadata_cache_get_metadata (BmpMetadataCache *self, const gchar *uri);

gchar*
bmp_metadata_cache_get_titlestring (BmpMetadataCache *self, const gchar *uri);

GHashTable*
bmp_metadata_cache_get_hash_value_table (BmpMetadataCache *self, const gchar *uri);
 
#endif /* METADATA_H */

