/* -*- mode: C; -*- */

%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

static GString *str = NULL;

%}

%option noyywrap

%x string

DIGIT         [0-9]
LETTER        [a-zA-Z]
ALPHANUM      {DIGIT}|{LETTER}
OPERATORS     "+"|"-"|"*"|"/"|"="|"<"|">"
SIGN          "+"|"-"

%%

{DIGIT}+ {
  /* Integer */
  printf ("Integer: %d\n", atoi (yytext));
}

{DIGIT}*"."{DIGIT}*"e"({SIGN}?{DIGIT}+)? {
  /* Real number */
  printf ("Float number: %f\n", atof (yytext));
}

({ALPHANUM}|{OPERATORS})+ {
  printf ("Identifier: %s\n", yytext);
}

"'" {
  /* Quote */
  printf ("Quote: %s\n", yytext);
}

";".*$ {
/* Comment */
  printf ("Comment: %s\n", yytext + 1);
}

"(" {
  /* Left parenthesis */
  printf ("Left parenthesis: %s\n", yytext);
}

")" {
  /* Right parenthesis */
  printf ("Right parenthesis: %s\n", yytext);
}

\" {
  BEGIN(string);

  str = g_string_new ("");
}

<string>[^\"\n]* {
  g_string_append (str, yytext);
}

<string>\n {
  printf ("Missing quote\n");
  BEGIN(INITIAL);
}

<string>\" {
  printf ("String: \"%s\"\n", str->str);
  g_string_free (str, TRUE);

  BEGIN(INITIAL);
}

[ \t\n]+ {
  /* whitespace */
  printf ("Whitespaces: %d\n", yyleng);
}

. {
  printf ("Unrecognized character: '%c'\n", yytext[0]);
}

%%
