#ifndef BMP_UI_H
#define BMP_UI_H

#include <gtk/gtk.h>

#include "ui_main.h"
#include "ui_playlist.h"
#include "ui_prefswin.h"
#include "ui_callbacks.h"
#include "bmp_window.h"

#include <glib-object.h>

#define BMP_TYPE_UI		  (bmp_ui_get_type ())
#define BMP_UI(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_UI, BmpUI))
#define BMP_UI_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_UI, BmpUIClass))
#define BMP_IS_UI(obj)	  	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_UI))
#define BMP_IS_UI_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_UI))
#define BMP_UI_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_UI, BmpUIClass))

typedef struct _BmpUI BmpUI;
typedef struct _BmpUIClass BmpUIClass;
typedef struct _BmpUIPrivate BmpUIPrivate;

struct _BmpUI {
    GObject parent;
    /*
     * instance members 
     */
    BmpUIPrivate *private;
};

struct _BmpUIClass {
    GObjectClass parent;

};

#define SET_CURSOR_FOR_WIDGET(cursor_enum,widget) \
  { \
    GdkCursor *cursor; \
      if (!GTK_WIDGET_REALIZED(widget)) gtk_widget_realize(GTK_WIDGET(widget)); \
      cursor = bmp_cursor_get(cursor_enum); \
      gdk_window_set_cursor (GTK_WIDGET(widget)->window, cursor); \
      gdk_cursor_destroy (cursor); \
  }

#define BMP_DEFAULT_FONT		"Sans 10"
#define BMP_DEFAULT_SKIN		"default"

#define BMP_MAIN_ACTION_GROUP		"action_group_main"
#define BMP_MAIN_ACTION_GROUP_TOGGLE    "action_group_main_toggle"

#define BMP_TOGGLE_ACTION_SHOW_UI "bmp_toggle_action_show_ui"
#define BMP_TOGGLE_ACTION_SHUFFLE "bmp_toggle_action_shuffle"
#define BMP_TOGGLE_ACTION_REPEAT  "bmp_toggle_action_repeat"
#define BMP_TOGGLE_ACTION_EQ	  "bmp_toggle_action_eq"
#define BMP_TOGGLE_ACTION_PL	  "bmp_toggle_action_pl"
#define BMP_TOGGLE_ACTION_SCROLL  "bmp_toggle_action_scroll"
#define BMP_TOGGLE_ACTION_STOP_AFTER_CURRENT "bmp_toggle_action_stop_after_current"
#define BMP_TOGGLE_ACTION_KEEP_ABOVE  "bmp_toggle_action_keep_above"

#define        BMP_ACTION_EJECT "bmp_action_eject"
#define        BMP_ACTION_PREV  "bmp_action_prev"
#define        BMP_ACTION_PLAY  "bmp_action_play"
#define        BMP_ACTION_PAUSE "bmp_action_pause"
#define        BMP_ACTION_STOP  "bmp_action_stop"
#define        BMP_ACTION_NEXT  "bmp_action_next"

#define	       BMP_ACTION_SORT_VISIBLE  "bmp_sort_visible"
#define	       BMP_ACTION_SORT_ARTIST   "bmp_sort_artist"
#define	       BMP_ACTION_SORT_ALBUM    "bmp_sort_album"
#define	       BMP_ACTION_SORT_TITLE    "bmp_sort_title"
#define	       BMP_ACTION_SORT_GENRE    "bmp_sort_genre"
#define	       BMP_ACTION_SORT_TRACK    "bmp_sort_track"
#define	       BMP_ACTION_SORT_LOCATION "bmp_sort_location"

#define        BMP_ACTION_PREFERENCES "bmp_action_preferences"

#define	       BMP_ACTION_CLEAR_PLAYBACK_HISTORY "bmp_action_clear_playback_history"

#define	       BMP_ACTION_ICONIFY "bmp_action_iconify"
#define	       BMP_ACTION_QUIT    "bmp_action_quit"
#define	       BMP_ACTION_ABOUT   "bmp_action_about"

#define	       BMP_ACTION_PLAYLIST_ADD          "playlist_add"
#define	       BMP_ACTION_PLAYLIST_DELETE       "playlist_del"
#define	       BMP_ACTION_PLAYLIST_DELETE_ALL   "playlist_del_all"
#define	       BMP_ACTION_PLAYLIST_RENAME       "playlist_rename"
#define	       BMP_ACTION_PLAYLIST_SET_FILENAME "playlist_set_filename"
#define	       BMP_ACTION_PLAYLIST_EXPORT	"playlist_export"

#define	       BMP_ACTION_TRACKLIST_ADD_URL     "tracklist_add_url"
#define	       BMP_ACTION_TRACKLIST_ADD_CD      "tracklist_add_cd"
#define	       BMP_ACTION_TRACKLIST_ADD_TO_NEW  "tracklist_add_to_new"
#define        BMP_ACTION_TRACKLIST_ADD         "tracklist_add_files"
#define        BMP_ACTION_TRACKLIST_ADD_LIBRARY "tracklist_add_library"
#define        BMP_ACTION_TRACKLIST_ADD_LIB_NEW "tracklist_add_library_new"
#define	       BMP_ACTION_TRACKLIST_DEL         "tracklist_del_files"
#define	       BMP_ACTION_TRACKLIST_CROP	"tracklist_crop_files"
#define        BMP_ACTION_TRACKLIST_DEL_ALL     "tracklist_del_files_all"
#define	       BMP_ACTION_TRACKLIST_FILEINFO	"tracklist_fileinfo"

#define	       BMP_ACTION_TRACKLIST_SELECT_ALL     "tracklist_select_all"
#define	       BMP_ACTION_TRACKLIST_SELECT_NONE    "tracklist_select_none"
#define	       BMP_ACTION_TRACKLIST_SELECT_INVERT  "tracklist_select_invert"

#define        BMP_FILE_REGION  "region.txt"
#define	       BMP_FILE_PLEDIT  "pledit.txt"

#define        BMP_MASK_REGION_SECTION_MAIN_WINDOW "normal"
#define        BMP_MASK_REGION_SECTION_EQUALIZER   "equalizer"

#define	       BMP_SLIDER_DEFAULT_STEP 4

typedef enum {
    SKIN_COMPONENT_MAIN,
    SKIN_COMPONENT_SHUFREP,
    SKIN_COMPONENT_CBUTTONS,
    SKIN_COMPONENT_POSBAR,
    SKIN_COMPONENT_VOLUME,
    SKIN_COMPONENT_TITLEBAR,
    SKIN_COMPONENT_BALANCE,
    SKIN_COMPONENT_PLEDIT,
    SKIN_COMPONENT_MONOSTER,
    SKIN_COMPONENT_NUMBERS,
    SKIN_COMPONENT_PLAYPAUSE,

    N_SKIN_COMPONENTS
} BmpSkinComponent;

enum {
    BMP_BUTTON_TOGGLE_STATE_DISABLED,
    BMP_BUTTON_TOGGLE_STATE_PRESSED,
    BMP_BUTTON_TOGGLE_STATE_ENABLED
};

enum {
	BMP_FONT_NORMAL,
	BMP_FONT_LARGE,

	BMP_N_FONTS
};

typedef struct {
	PangoFontDescription *font_description;

	gint font_ascent;
	gint font_descent;
	gint font_height;

	gboolean font_has_slant;

	gint font_padding;
	gint font_width_approx_letters, font_width_delta_digit_one;
	gint font_width_colon, font_width_colon_third;
	gint font_width_approx_digits, font_width_approx_digits_half;
} BmpFontInformation;
extern BmpFontInformation* bmp_fonts[BMP_N_FONTS];

typedef struct {
    gchar *widget_name;
    gchar *tooltip_text;
    gchar *stock_id;
} BmpTooltipEntry;

typedef struct _bmp_button_normal_t bmp_button_normal_t;
struct _bmp_button_normal_t {
    gchar *action_id;
    gchar *tooltip;
    gint pixmap_id;             /* id of used pixmap */
    gint img_x;
    gint img_y;
    gint img_x_pressed;
    gint img_y_pressed;
    gint width;
    gint height;
    gint pos_x;
    gint pos_y;
};

typedef struct _bmp_button_toggle_t bmp_button_toggle_t;
struct _bmp_button_toggle_t {
    gchar *action_id;
    gchar *tooltip;
    gint pixmap_id;             /* id of used pixmap */
    gint img_x;
    gint img_y;
    gint img_x_pressed;
    gint img_y_pressed;
    gint img_x_active;
    gint img_y_active;
    gint width;                 /* width in .bmp file */
    gint height;                /* height in .bmp file */
    gint pos_x;                 /* x position in skin when displayed */
    gint pos_y;                 /* y position in skin when displayed */
    gboolean clickable;         /* is it clickable? */
};

typedef struct _bmp_skin_component_t bmp_skin_component_t;
struct _bmp_skin_component_t {
    BmpSkinComponent identifier;
    gchar *filename;
    GdkPixbuf *pixbuf;
    gpointer user_data;
};

typedef struct {
    GtkWidget *window;
} bmp_window_t;

typedef struct {
    BmpSkinComponent identifier;
    gchar *filename;
    GdkPixbuf *pixbuf;
    gint x_slider;
    gboolean dragging;
} bmp_posbar_t;

typedef struct {
    BmpSkinComponent identifier;
    gchar *filename;
    GdkPixbuf *pixbuf;
    gint x_slider;
    gint height;
    gboolean dragging;
    gboolean has_button;
} bmp_volume_t;

#define BMP_WINDOW_MAIN_WIDTH 275
#define BMP_WINDOW_MAIN_HEIGHT 116

enum {
    BMP_SKIN_ARCHIVE,
    BMP_SKIN_DIR
};

enum {
    BMP_WINDOW_MAIN,
    BMP_WINDOW_PLEDIT,
    BMP_WINDOW_EQ
};

typedef enum {
    BMP_CHROMA_VSCROLLBAR_PLAYLISTS,
    BMP_CHROMA_VSCROLLBAR_TRACKLIST,

    BMP_CHROMA_LIST_TRACKLIST,
    BMP_CHROMA_LIST_PLAYLISTS,

    BMP_N_CHROMA_WIDGETS
} BmpChromaWidgets;
#define BMP_CHROMA_WIDGET_NONE -1

typedef enum {
    BMP_CHROMA_WIDGET_TYPE_VSCROLLBAR,
    BMP_CHROMA_WIDGET_TYPE_LIST,
    BMP_CHROMA_WIDGET_TYPE_BUTTON
} BmpChromaWidgetType;
gint chroma_active_widget;

typedef struct {
    GObject *widget;
    gint type;
} BmpChromaWidget;

extern BmpChromaWidget *chroma_widgets[BMP_N_CHROMA_WIDGETS];

extern GtkUIManager    *bmp_ui_manager;
extern GtkActionGroup  *bmp_actions;
extern GtkActionGroup  *bmp_toggle_actions;

extern bmp_posbar_t *bmp_posbar;
extern bmp_volume_t *bmp_volume;
extern bmp_volume_t *bmp_balance;
extern bmp_skin_component_t *bmp_skin_component[N_SKIN_COMPONENTS];

typedef struct {
    double x;
    double y;
} BmpPoint;

/* API */
GType
bmp_ui_get_type(void);

BmpUI*
bmp_ui_new(void);

gint
bmp_ui_finalize_init (BmpUI *self);
 
GdkBitmap*
bmp_ui_get_mask_window_main (BmpUI *self);

void
bmp_ui_set_trayicon_text (BmpUI *self, const gchar *text);

void
bmp_ui_set_trayicon_exit (BmpUI *self);

#endif
