#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <glib/gi18n.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>

#include "main.h"

#include "ui.h"
#include "ui_util.h"

#include "gtkhsv.h"

#include "chroma_list.h"
#include "g_object_get.h"

#include "bmpx/util.h"

#define _NET_WM_STATE_REMOVE   0
#define _NET_WM_STATE_ADD      1
#define _NET_WM_STATE_TOGGLE   2

#define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
#define _NET_WM_MOVERESIZE_SIZE_TOP          1
#define _NET_WM_MOVERESIZE_SIZE_TOPRIGHT     2
#define _NET_WM_MOVERESIZE_SIZE_RIGHT        3
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT  4
#define _NET_WM_MOVERESIZE_SIZE_BOTTOM       5
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT   6
#define _NET_WM_MOVERESIZE_SIZE_LEFT         7
#define _NET_WM_MOVERESIZE_MOVE              8

static gchar    *bmp_skin_colors[BMP_N_SKINCOLORS];
static GdkColor *bmp_skin_colors_gdk[BMP_N_SKINCOLORS];
static GQuark quark_popup_data;

/* Cursors */
const gchar * _bmp_cursor_names[] = {
   "normal.cur",
   "posbar.cur",
	"volbar.cur",
	"pnormal.cur",
	"psize.cur",
	"mainmenu.cur"
};
static GdkPixbuf *bmp_cursors[CURSOR_N_CURSORS];


static gchar* bmp_skin_color_names[] = {
   "NormalBG",
   "Normal",
   "SelectedBG",
   "Current"
};

GdkPixbuf*
bmp_pixbuf_new_simple(guint width, guint height) {
  return gdk_pixbuf_new (gdk_pixbuf_get_colorspace(bmp_skin_component[SKIN_COMPONENT_PLEDIT]->pixbuf),
                                            gdk_pixbuf_get_has_alpha(bmp_skin_component[SKIN_COMPONENT_PLEDIT]->pixbuf),
                                            gdk_pixbuf_get_bits_per_sample(bmp_skin_component[SKIN_COMPONENT_PLEDIT]->pixbuf),
                                            width, height);
}

void
bmp_pixbuf_copy_area_simple(GdkPixbuf *dest, guint x, guint y, guint width, guint height) {
  gdk_pixbuf_copy_area (bmp_skin_component[SKIN_COMPONENT_PLEDIT]->pixbuf, x, y, width, height, dest, 0, 0);
}

void
bmp_draw_pixbuf_simple(cairo_t *cr, GdkPixbuf *buf, guint x, guint y) {
    cairo_save (cr);
    gdk_cairo_set_source_pixbuf (cr,
				 buf,
				 x,
				 y);				 
    cairo_paint (cr);
    cairo_restore (cr);
}

/* CURSORS */
void
bmp_cursors_init(void)
{
  bmp_cursors[0] = NULL;
}

void
bmp_cursors_free(void)
{
  gint n;

  if (bmp_cursors[0] != NULL)
	for (n = 0; n < CURSOR_N_CURSORS; n++) 
	  {
	     if (bmp_cursors[n] != NULL) 
	       {
        	 g_object_unref (bmp_cursors[n]);
	       }
	     bmp_cursors[n] = NULL;
	  }
}

void
bmp_cursors_set (const gchar *archive_dirname) 
{
  /* Cursors */
  bmp_cursor_add(archive_dirname, CURSOR_NORMAL);
  bmp_cursor_add(archive_dirname, CURSOR_POSBAR);
  bmp_cursor_add(archive_dirname, CURSOR_VOLBAR);
  bmp_cursor_add(archive_dirname, CURSOR_PLEDIT);
  bmp_cursor_add(archive_dirname, CURSOR_PLEDIT_SIZE);
  bmp_cursor_add(archive_dirname, CURSOR_MAINMENU);
}

static GdkPixbuf*
get_cursor_pixbuf (const gchar *archive_dirname, const gchar *cursor_filename)
{
  GdkPixbuf *pb_aux, *pb_return; 
  GdkPixbufAnimation * cursor_animated;
  gchar * filename = NULL;

  filename = find_file_recursively(archive_dirname, cursor_filename, 1);
 
  if (!filename) return NULL;

  cursor_animated = 
	gdk_pixbuf_animation_new_from_file (filename, NULL);
  pb_aux = 
	gdk_pixbuf_animation_get_static_image (cursor_animated);
  pb_return = 
	gdk_pixbuf_copy (pb_aux);

 g_free (filename);
 g_object_unref (cursor_animated);
 return pb_return;
}

void
bmp_cursor_add (const gchar *archive_dirname, gint cursor_enum) {

  GdkPixbuf * pixbuf = NULL;

  pixbuf = get_cursor_pixbuf(archive_dirname, _bmp_cursor_names[cursor_enum]); 

  bmp_cursors[cursor_enum] = pixbuf;
}


GdkCursor*
bmp_cursor_get (gint cursor_enum) {

	GdkCursor * cursor;
	GdkPixbuf * pixbuf;
	const gchar *hot_x = NULL, *hot_y = NULL;
	gint hot_x_int = 0, hot_y_int = 0;

	pixbuf = bmp_cursors[cursor_enum];

	if (!bmp_configuration_key_fetch_boolean(bmp_configuration,"bmp","use-custom-cursors"))
	{
			cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_LEFT_PTR);
			return cursor;
	}

	if (!pixbuf) 
	{
		pixbuf = bmp_cursors[CURSOR_NORMAL];

		if (!pixbuf)
		{
			cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_LEFT_PTR);
			return cursor;
		}
	}

	hot_x = gdk_pixbuf_get_option(pixbuf,"x_hot");
	hot_y = gdk_pixbuf_get_option(pixbuf,"y_hot");

	if (hot_x) hot_x_int = strtol (hot_x, NULL, 10); 
	if (hot_y) hot_y_int = strtol (hot_x, NULL, 10); 

        cursor = gdk_cursor_new_from_pixbuf  (gdk_display_get_default(),
                                              pixbuf, 
                                              hot_x_int,
                                              hot_y_int);
	return cursor;
}
/* END cursors */

/* XShape Masks */
static GdkBitmap*
bmp_skin_create_transparent_mask(const gchar *path,
                                 const gchar *file,
                                 const gchar *section,
                                 gint width,
                                 gint height)
{
    GdkBitmap *mask = NULL;
    GdkGC *gc = NULL;
    GdkColor pattern;
    GdkPoint *gpoints;

    gchar *filename = NULL;
    gboolean created_mask = FALSE;
    GArray *num, *point;
    gint i, j, k;

    if (path)
        filename = (gchar*)find_file_recursively(path, file, 1);

    /* filename will be null if path wasn't set */
    if (!filename)
      {
/*        return create_default_mask (window, width, height);  */
	return NULL;
      }

    if ((num = (GArray*)read_ini_array(filename, section, "NumPoints")) == NULL)
      {
        g_free(filename);
        return NULL;
      }

    if ((point = (GArray*)read_ini_array(filename, section, "PointList")) == NULL)
      {
        g_array_free(num, TRUE);
        g_free(filename);
        return NULL;
      }

    mask = gdk_pixmap_new(NULL, width, height, 1);
    gc = gdk_gc_new(mask);

    pattern.pixel = 0;
    gdk_gc_set_foreground(gc, &pattern);
    gdk_draw_rectangle(mask, gc, TRUE, 0, 0, width, height);
    pattern.pixel = 1;
    gdk_gc_set_foreground(gc, &pattern);

    j = 0;
    for (i = 0; i < num->len; i++) {
        if ((point->len - j) >= (g_array_index(num, gint, i) * 2)) {
            created_mask = TRUE;
            gpoints = g_new(GdkPoint, g_array_index(num, gint, i));
            for (k = 0; k < g_array_index(num, gint, i); k++) {
                gpoints[k].x = g_array_index(point, gint, j + k * 2);
                gpoints[k].y = g_array_index(point, gint, j + k * 2 + 1);
            }
            j += k * 2;
            gdk_draw_polygon(mask, gc, TRUE, gpoints,
                             g_array_index(num, gint, i));
            g_free(gpoints);
        }
    }
    g_array_free(num, TRUE);
    g_array_free(point, TRUE);
    g_free(filename);

    if (!created_mask)
        gdk_draw_rectangle(mask, gc, TRUE, 0, 0, width, height);

    gdk_gc_destroy(gc);

    return mask;
}

GdkBitmap*
bmp_get_mask_window_main (const gchar *path)
{
    return bmp_skin_create_transparent_mask(path,
                                 BMP_FILE_REGION, 
                                 BMP_MASK_REGION_SECTION_MAIN_WINDOW, 
                                 275, 
                                 116); 
}

static guint
hex_chars_to_int(gchar hi, gchar lo)
{
    /*
     * Converts a value in the range 0x00-0xFF
     * to a integer in the range 0-65535
     */
    gchar str[3];
    guint value;

    str[0] = hi;
    str[1] = lo;
    str[2] = 0;

    value = (CLAMP(strtol(str, NULL, 16), 0, 0xFF) << 8);

    return value;
}

#define PLEDIT_TEXT_SECTION_NAME "Text"

/* COLORS */
gchar*
bmp_skin_get_color(const gchar *path, const gchar *key) 
{
    gchar *filename, *value;

    filename = (gchar*)find_file_recursively(path, BMP_FILE_PLEDIT, 1);

    if (filename)
      {

	value = (gchar*)read_ini_string (filename, PLEDIT_TEXT_SECTION_NAME, key);
	g_free (filename);

	if (value)
	    return value;
        else 
	    return NULL;

      }

    return NULL;
}

GdkColor*
bmp_skin_convert_color(const gchar *value_arg)
{
    GdkColor *color = NULL;

        if (value_arg)
          {
	    gchar *value = g_strdup (value_arg);
            gchar *ptr = value; 
            gint len;

            color = g_new0(GdkColor, 1);
            g_strstrip(value);

            if (value[0] == '#')
                ptr++;
            len = strlen(ptr);

            /*
             * The handling of incomplete values is done this way
             * to maximize winamp compatibility
             */
            if (len >= 6) {
                color->red = hex_chars_to_int(*ptr, *(ptr + 1));
                ptr += 2;
            }
            if (len >= 4) {
                color->green = hex_chars_to_int(*ptr, *(ptr + 1));
                ptr += 2;
            }
            if (len >= 2)
                color->blue = hex_chars_to_int(*ptr, *(ptr + 1));

	    g_free (value);
        }
    
   return color;
}


void
bmp_colors_set_color(BmpSkinColor color, const gchar *text) {
    bmp_skin_colors[color] = g_strdup (text); 
}

void
bmp_colors_init(const gchar *archive_dirname) {

  gchar *aux;

  aux = bmp_skin_get_color(archive_dirname, bmp_skin_color_names[BMP_SKINCOLOR_PL_BG_NORMAL]); 
  bmp_colors_set_color(BMP_SKINCOLOR_PL_BG_NORMAL, aux); 
  g_free (aux);
  aux = bmp_skin_get_color(archive_dirname, bmp_skin_color_names[BMP_SKINCOLOR_PL_FG_CURRENT]); 
  bmp_colors_set_color(BMP_SKINCOLOR_PL_FG_CURRENT, aux); 
  g_free (aux);
  aux = bmp_skin_get_color(archive_dirname, bmp_skin_color_names[BMP_SKINCOLOR_PL_FG_NORMAL]); 
  bmp_colors_set_color(BMP_SKINCOLOR_PL_FG_NORMAL, aux); 
  g_free (aux);
  aux = bmp_skin_get_color(archive_dirname, bmp_skin_color_names[BMP_SKINCOLOR_PL_BG_SELECTED]); 
  bmp_colors_set_color(BMP_SKINCOLOR_PL_BG_SELECTED, aux); 
  g_free (aux);
}


void
bmp_colors_free(void) {
    gint i;

    for (i = 0; i < SKIN_N_COLORS; i++)
    {
        /* gdk_color_free(bmp_skin_colors_gdk[i]);     */
        bmp_skin_colors_gdk[i] = NULL;
    }
}

void
bmp_color_allocate(BmpSkinColor color, GtkWidget *widget)
{
	gdk_color_alloc(gdk_window_get_colormap(widget->window), bmp_colors_get_color(color));
}

GdkColor*
bmp_colors_get_color(BmpSkinColor color) {
	if (!bmp_skin_colors_gdk[color]) {
		bmp_skin_colors_gdk[color] = bmp_skin_convert_color(bmp_skin_colors[color]);
	}
	return bmp_skin_colors_gdk[color];
}

GdkColor*
bmp_colors_change_brightness(GdkColor *original, GtkWidget *widget, gfloat delta) {

    gdouble h,s,v, r,g,b;
    GdkColor *rv;

    rv = gdk_color_copy(original);
    r =  (gfloat) ((gfloat)(rv->red)/(gfloat)65535);
    g =  (gfloat) ((gfloat)(rv->green)/(gfloat)65535);
    b =  (gfloat) ((gfloat)(rv->blue)/(gfloat)65535);

    gtk_rgb_to_hsv(r,g,b, &h,&s,&v);

    v = v + delta;

    if (v>1) v=1;
    if (v<0) v=0;

    gtk_hsv_to_rgb(h,s,v,&r,&g,&b);
    rv->red = r*65535;
    rv->green = g*65535;
    rv->blue = b*65535;
    gdk_color_alloc(gdk_window_get_colormap(widget->window), rv);

    return rv;
}

void
gdk_color_to_rgba (GdkColor *color, double *r, double *g, double *b, double *a)
{
  *r = color->red   / 65535.0;
  *g = color->green / 65535.0;
  *b = color->blue  / 65535.0;
  *a = 1;
}
/* END colors */

void
draw_polygon (cairo_t *cr, GdkColor *color, double alpha, BmpPoint points[], gint n_points, gboolean fill)
{
	gint n;
	double r,g,b,a;

 	cairo_set_line_width (cr, 1.0);
	gdk_color_to_rgba (color, &r, &g, &b, &a);

	cairo_move_to (cr, points[0].x, points[0].y);

	for (n = 1; n < n_points; n++)
		cairo_line_to (cr, points[n].x, points[n].y);

	cairo_line_to (cr, points[0].x, points[0].y);
	cairo_set_source_rgba (cr, r, g, b, alpha);

	if (fill)
		cairo_fill (cr);
	else
		cairo_stroke (cr);

	cairo_fill (cr);
}

void
bmp_draw_rectangle_simple (cairo_t *cr, GdkColor *color, double alpha, gint x, gint y, gint width, gint height)
{
    double r, g, b, a;

    cairo_rectangle (cr, 
			     x,
		             y,
		 	     width, 
			     height);
    gdk_color_to_rgba (color, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, alpha); 
    cairo_fill (cr);
}

static void
_draw_rounded_rectangle (cairo_t *cr, double xc, double yc, double _w, double _h, GdkColor *color, double alpha, double line_width, gboolean fill, gdouble rounding_offset)
{
    double r, g, b, a;

    cairo_set_line_width (cr, line_width);

    cairo_move_to (cr, xc, yc+(rounding_offset*2));

    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc,
				xc+rounding_offset, yc);

    cairo_line_to  (cr, xc+(_w/2), yc);

    cairo_curve_to (cr, xc+_w-rounding_offset, yc,
			xc+_w, yc,
			xc+_w, yc+rounding_offset);

    cairo_line_to (cr,  xc+_w, yc+(_h/2));

    cairo_curve_to (cr, xc+_w, yc+_h-rounding_offset,
		        xc+_w, yc+_h,
			xc+_w-rounding_offset, yc+_h);

    cairo_line_to (cr, xc+(_w/2), yc+_h);

    cairo_curve_to (cr, xc+rounding_offset, yc+_h,
			xc, yc+_h,
			xc, yc+_h-rounding_offset);


    cairo_close_path (cr);
    gdk_color_to_rgba (color, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, alpha);

    if (fill)
	    cairo_fill (cr);
    else
	    cairo_stroke (cr);
}

void
draw_rounded_rectangle_pattern (cairo_t *cr, 
					    double xc, 
					    double yc, 
					    double _w, 
					    double _h, 
					    cairo_pattern_t *cr_pattern, 
					    double line_width, 
					    gboolean fill, 
					    gdouble rounding_offset)
{
    cairo_set_line_width (cr, line_width);

    cairo_move_to (cr, xc, yc+(rounding_offset*2));

    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc,
				xc+rounding_offset, yc);

    cairo_line_to  (cr, xc+_w-(rounding_offset*2), yc);

    cairo_curve_to (cr, xc+_w-rounding_offset, yc,
			xc+_w, yc,
			xc+_w, yc+rounding_offset);

    cairo_line_to (cr,  xc+_w, yc+_h-(rounding_offset*2));

    cairo_curve_to (cr, xc+_w, yc+_h-rounding_offset,
		        xc+_w, yc+_h,
			xc+_w-rounding_offset, yc+_h);

    cairo_line_to (cr, xc+(rounding_offset*2), yc+_h);

    cairo_curve_to (cr, xc+rounding_offset, yc+_h,
			xc, yc+_h,
			xc, yc+_h-rounding_offset);


    cairo_close_path (cr);
    cairo_set_source (cr, cr_pattern); 
    if (fill)
      cairo_fill (cr);
    else
      cairo_stroke (cr);
}



void
draw_rounded_rectangle (cairo_t *cr, 
				    double xc, 
				    double yc, 
				    double _w, 
				    double _h, 
				    GdkColor *color, 
				    double alpha, 
				    double line_width, 
				    gboolean fill, 
				    gdouble rounding_offset)
{
    double r, g, b, a;

    cairo_set_line_width (cr, line_width);

    cairo_move_to (cr, xc, yc+(rounding_offset*2));

    cairo_curve_to (cr, 	xc, yc+rounding_offset,
				xc, yc,
				xc+rounding_offset, yc);

    cairo_line_to  (cr, xc+_w-(rounding_offset*2), yc);

    cairo_curve_to (cr, xc+_w-rounding_offset, yc,
			xc+_w, yc,
			xc+_w, yc+rounding_offset);

    cairo_line_to (cr,  xc+_w, yc+_h-(rounding_offset*2));

    cairo_curve_to (cr, xc+_w, yc+_h-rounding_offset,
		        xc+_w, yc+_h,
			xc+_w-rounding_offset, yc+_h);

    cairo_line_to (cr, xc+(rounding_offset*2), yc+_h);

    cairo_curve_to (cr, xc+rounding_offset, yc+_h,
			xc, yc+_h,
			xc, yc+_h-rounding_offset);


    cairo_close_path (cr);
    gdk_color_to_rgba (color, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, alpha);

    if (fill)
      cairo_fill (cr);
    else
      cairo_stroke (cr);
}

void
bmp_draw_rectangle_rounded (cairo_t *cr, GdkColor *fg, GdkColor *bg, GdkColor *shadow, gint x, gint y, gint width, gint height, gboolean pushed)
{
   _draw_rounded_rectangle (cr, x+1, y+1, width, height, shadow, 0.8, 1.5, TRUE, 0.65);
   _draw_rounded_rectangle (cr, x  , y  , width, height, fg    , 1.0, 1.5, TRUE, 0.65);
}


void
bmp_draw_rectangle (cairo_t *cr, GdkColor *fg, GdkColor *bg, GdkColor *shadow, gint x, gint y, gint width, gint height, gboolean pushed)
{
    gint offset = 0;
    double r, g, b, a;

    if (pushed) offset = 1; 

    if (!pushed)
    {
	cairo_rectangle (cr, x + offset + 1,
		             y + offset + 1,
		 	     width, height);
	gdk_color_to_rgba (shadow, &r, &g, &b, &a);
        cairo_set_source_rgba (cr, r, g, b, 0.9);
	cairo_fill (cr);
    }

    cairo_rectangle (cr, 
			     x + offset,
		             y + offset,
		 	     width, 
			     height);
    gdk_color_to_rgba (fg, &r, &g, &b, &a);
    cairo_set_source_rgba (cr, r, g, b, 1); 
    cairo_fill (cr);
}

BmpFontInformation*
bmp_get_font_information_pfd (const PangoFontDescription *font_desc, GtkWidget *widget)
{
   BmpFontInformation *fi;
   PangoFontMetrics *metrics;
   PangoContext *context;
   gdouble resolution_scale;

   /* FIXME: Display resolution... */

   fi = g_new0(BmpFontInformation,1);
   fi->font_description = pango_font_description_copy (font_desc);

#if 0
   metrics = pango_context_get_metrics (gdk_pango_context_get_for_screen(gdk_screen_get_default()),
				        fi->font_description, 
				        NULL);
#endif

   context = gtk_widget_get_pango_context (widget);

   metrics = pango_context_get_metrics (context,
				        fi->font_description, 
				        pango_context_get_language(context));


   fi->font_ascent  = pango_font_metrics_get_ascent (metrics)/PANGO_SCALE;
   fi->font_descent = pango_font_metrics_get_descent (metrics)/PANGO_SCALE;
//   fi->font_height = fi->font_ascent /*+ abs(fi->font_descent)*/;

   fi->font_height = fi->font_ascent + fi->font_descent;

   resolution_scale = (screen_get_resolution (gdk_screen_get_default())/96.0);

   fi->font_width_approx_letters = (pango_font_metrics_get_approximate_char_width (metrics)/PANGO_SCALE);
   fi->font_width_approx_digits  = (pango_font_metrics_get_approximate_digit_width (metrics)/PANGO_SCALE)*resolution_scale;

   pango_font_metrics_unref (metrics);

   return fi;
}

BmpFontInformation*
bmp_get_font_information (const gchar *fontname, GtkWidget *widget)
{
    BmpFontInformation *font_information;
    PangoFontDescription *font_desc;

    g_return_val_if_fail(fontname != NULL, FALSE);

    font_desc = 
	pango_font_description_from_string (fontname);
    font_information = 
	bmp_get_font_information_pfd (font_desc, widget); 

    pango_font_description_free (font_desc);
    return font_information;
}

#define BMP_N_ICON_SIZES 5 
void
bmp_window_set_icon_list (GtkWidget *window, const gchar *icon_name)
{
	static const gchar* sizes[] = { 
		"16", "32", "48", "64", "128"
	};

	GdkPixbuf *pixbufs[BMP_N_ICON_SIZES];
	GError *error = NULL;
	GList *icon_list = NULL;
	const gchar *icon_theme;
	gchar *aux0, *aux1;
	gint n;

	icon_theme = bmp_configuration_key_fetch (bmp_configuration, "bmp", "icon-theme");

	for (n = 0; n < BMP_N_ICON_SIZES; n++)
	  {
		aux0 = g_strconcat("icon_", icon_name, "_", sizes[n], ".png", NULL); 
		aux1 = g_build_filename (DATA_DIR, BMP_ICONS_BASENAME, icon_theme, aux0, NULL);

		pixbufs[n] = gdk_pixbuf_new_from_file (aux1, &error);

		if (error != NULL)
		  {
			g_message (error->message);
			g_error_free (error);
			error = NULL;
		  }

		icon_list = g_list_append (icon_list, pixbufs[n]);
		g_free (aux1);
		g_free (aux0);
	  }

	gtk_window_set_icon_list (GTK_WINDOW (window), icon_list);
	g_list_free (icon_list);

	for (n = 0; n < BMP_N_ICON_SIZES; n++)
		g_object_unref (pixbufs[n]);
}

static gdouble
mm_to_inch (gdouble mm)
{
    return mm / 25.40;
}

gdouble
screen_get_resolution (GdkScreen *screen)
{
  /* NOTE: this assumes the width and height resolutions are
     equal. This is only true for video modes with aspect ratios of
     4:3 e.g. 640x480, 800x600, etc. - descender */
  return ceil(screen_get_y_resolution (screen));
}

gdouble
screen_get_x_resolution (GdkScreen *screen)
{
  g_return_val_if_fail (GDK_IS_SCREEN (screen), -1.0);

  return (gdouble) gdk_screen_get_width (screen) / mm_to_inch (gdk_screen_get_height_mm (screen));
}

gdouble
screen_get_y_resolution (GdkScreen *screen)
{
  g_return_val_if_fail (GDK_IS_SCREEN (screen), -1.0);

  return (gdouble) gdk_screen_get_height (screen) / mm_to_inch (gdk_screen_get_height_mm (screen));
}


GtkWidget *
ui_manager_get_popup (GtkUIManager * self,
		      const gchar * path)
{
    GtkWidget *menu_item;
    gpointer submenu;

    menu_item = gtk_ui_manager_get_widget (self, path);

    if (GTK_IS_MENU_ITEM (menu_item))
	submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
    else
	submenu = NULL;

    if (submenu)
	{
	    SET_CURSOR_FOR_WIDGET (CURSOR_NORMAL, GTK_WIDGET (submenu));
	}

    return submenu;
}

static void
menu_popup_pos_func (GtkMenu * menu,
		     gint * x,
		     gint * y,
		     gboolean * push_in,
		     gint * point)
{
    *x = point[0];
    *y = point[1];
    *push_in = FALSE;
}

void
bmp_menu_popup (GtkMenu * menu,
		gint x,
		gint y,
		guint button,
		guint time)
{
    gint pos[2];

    pos[0] = x;
    pos[1] = y;

    gtk_menu_popup (menu, NULL, NULL,
		    (GtkMenuPositionFunc) menu_popup_pos_func, pos,
		    button, time);
}

typedef struct {
    gint x;
    gint y;
} MenuPos;

void
util_menu_position (GtkMenu * menu,
		    gint * x,
		    gint * y,
		    gboolean * push_in,
		    gpointer data)
{
    GtkRequisition requisition;
    gint screen_width;
    gint screen_height;
    MenuPos *pos = data;

    gtk_widget_size_request (GTK_WIDGET (menu), &requisition);

    screen_width = gdk_screen_width ();
    screen_height = gdk_screen_height ();

    *x = CLAMP (pos->x - 2, 0, MAX (0, screen_width - requisition.width));
    *y = CLAMP (pos->y - 2, 0, MAX (0, screen_height - requisition.height));
}

static void util_menu_delete_popup_data (GtkObject * object,
					 GtkItemFactory * ifactory)
{
    gtk_signal_disconnect_by_func (object,
				   GTK_SIGNAL_FUNC
				   (util_menu_delete_popup_data), ifactory);
    gtk_object_remove_data_by_id (GTK_OBJECT (ifactory), quark_popup_data);
}

#ifdef ENABLE_NLS
gchar *
bmp_menu_translate (const gchar *message,
                    gpointer     data)
{
  gchar *translation = (gchar *) Q_(message);

  if (!translation)
    {
      g_warning ("Bad translation for message: %s", message);
      translation = (gchar *) message;
    }

  return translation;
}
#endif

void
util_set_cursor (GtkWidget * window)
{
    static GdkCursor *cursor;

    if (! window)
	{
	    if (cursor)
		{
		    gdk_cursor_destroy (cursor);
		    cursor = NULL;
		}
	    return;
	}
    if (! cursor)
	cursor = gdk_cursor_new (GDK_LEFT_PTR);

    gdk_window_set_cursor (window->window, cursor);
}


