#include <config.h>

#ifdef HAVE_GUI
#include <gdk/gdkprivate.h>
#include <gtk/gtkimage.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>
#include <X11/Xlib.h>
#include "ui.h"
#include "ui_util.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <string.h>
#include <strings.h>

#include <glib/gstdio.h>
#include <glib/gi18n.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#include "main.h"
#include "configuration.h"

#include "bmpx/uri.h"
#include "bmpx/util.h"
#include "bmpx/metadata.h"

struct _BmpUniqueList
{
  GList *list;
  GHashTable *hash;
};

static GArray *string_to_garray (const gchar *str);

gchar*
read_ini_string_buffer (const gchar * _buffer, const gchar * section,
			const gchar * key)
{
    gchar *ret_buffer = NULL;
    gint found_section = 0, off = 0, len = 0;
    gsize filesize;
    gchar *buffer = NULL;

    if (! _buffer)
	return NULL;

    buffer = g_strdup (_buffer);

    filesize = strlen (buffer);

    while (! ret_buffer && off < filesize)
	{
	    while (off < filesize &&
		   (buffer[off] == '\r' || buffer[off] == '\n' ||
		    buffer[off] == ' ' || buffer[off] == '\t'))
		off++;
	    if (off >= filesize)
		break;
	    if (buffer[off] == '[')
		{
		    gint slen = strlen (section);
		    off++;
		    found_section = 0;
		    if (off + slen + 1 < filesize &&
			!strncasecmp (section, &buffer[off], slen))
			{
			    off += slen;
			    if (buffer[off] == ']')
				{
				    off++;
				    found_section = 1;
				}
			}
		}
	    else if (found_section && off + strlen (key) < filesize &&
		     !strncasecmp (key, &buffer[off], strlen (key)))
		{
		    off += strlen (key);
		    while (off < filesize &&
			   (buffer[off] == ' ' || buffer[off] == '\t'))
			off++;
		    if (off >= filesize)
			break;
		    if (buffer[off] == '=')
			{
			    off++;
			    while (off < filesize &&
				   (buffer[off] == ' ' || buffer[off] == '\t'))
				off++;
			    if (off >= filesize)
				break;
			    len = 0;
			    while (off + len < filesize &&
				   buffer[off + len] != '\r' &&
				   buffer[off + len] != '\n' &&
				   buffer[off + len] != ';')
				len++;
			    ret_buffer = g_strndup (&buffer[off], len);
			    off += len;
			}
		}
	    while (off < filesize &&
		   buffer[off] != '\r' && buffer[off] != '\n')
		off++;
	}

    return ret_buffer;
}


gchar*
read_ini_string (const gchar * filename, const gchar * section,
		 const gchar * key)
{
    gchar *buffer, *ret_buffer = NULL;
    gint found_section = 0, off = 0, len = 0;
    gsize filesize;

    if (! filename)
	return NULL;

    if (! g_file_get_contents (filename, &buffer, &filesize, NULL))
	return NULL;

    while (! ret_buffer && off < filesize)
	{
	    while (off < filesize &&
		   (buffer[off] == '\r' || buffer[off] == '\n' ||
		    buffer[off] == ' ' || buffer[off] == '\t'))
		off++;
	    if (off >= filesize)
		break;
	    if (buffer[off] == '[')
		{
		    gint slen = strlen (section);
		    off++;
		    found_section = 0;
		    if (off + slen + 1 < filesize &&
			! strncasecmp (section, &buffer[off], slen))
			{
			    off += slen;
			    if (buffer[off] == ']')
				{
				    off++;
				    found_section = 1;
				}
			}
		}
	    else if (found_section && off + strlen (key) < filesize &&
		     ! strncasecmp (key, &buffer[off], strlen (key)))
		{
		    off += strlen (key);
		    while (off < filesize &&
			   (buffer[off] == ' ' || buffer[off] == '\t'))
			off++;
		    if (off >= filesize)
			break;
		    if (buffer[off] == '=')
			{
			    off++;
			    while (off < filesize &&
				   (buffer[off] == ' ' || buffer[off] == '\t'))
				off++;
			    if (off >= filesize)
				break;
			    len = 0;
			    while (off + len < filesize &&
				   buffer[off + len] != '\r' &&
				   buffer[off + len] != '\n' &&
				   buffer[off + len] != ';')
				len++;
			    ret_buffer = g_strndup (&buffer[off], len);
			    off += len;
			}
		}
	    while (off < filesize &&
		   buffer[off] != '\r' && buffer[off] != '\n')
		off++;
	}

    g_free (buffer);
    return ret_buffer;
}

static GArray*
string_to_garray (const gchar * str)
{
    GArray *array;
    gint temp;
    const gchar *ptr = str;
    gchar *endptr;

    array = g_array_new (FALSE, TRUE, sizeof (gint));
    for (;;)
	{
	    temp = strtol (ptr, &endptr, 10);
	    if (ptr == endptr)
		break;
	    g_array_append_val (array, temp);
	    ptr = endptr;
	    while (! isdigit (*ptr) && (*ptr) != '\0')
		ptr++;
	    if (*ptr == '\0')
		break;
	}
    return (array);
}

GArray *read_ini_array (const gchar * filename, const gchar * section,
			const gchar * key)
{
    gchar *temp;
    GArray *a;

    if ((temp = read_ini_string (filename, section, key)) == NULL)
	{
	    g_free (temp);
	    return NULL;
	}
    a = string_to_garray (temp);
    g_free (temp);
    return a;
}

void glist_movedown (GList * list)
{
    gpointer temp;

    if (g_list_next (list))
	{
	    temp = list->data;
	    list->data = list->next->data;
	    list->next->data = temp;
	}
}

void glist_moveup (GList * list)
{
    gpointer temp;

    if (g_list_previous (list))
	{
	    temp = list->data;
	    list->data = list->prev->data;
	    list->prev->data = temp;
	}
}

void
glist_swap (GList * list1, GList * list2)
{
    gpointer aux;

    aux = list1->data;

    list1->data = list2->data;
    list2->data = aux;
}

GList *
glist_make_shuffle_index (GList * list_in)
{
    gint len = g_list_length (list_in);
    gint i, j, n;
    GList *list = NULL, *node, **ptrs;

    if (! len)
	return NULL;

    for (n = 0; n < g_list_length (list_in); n++)
	list = g_list_append (list, GINT_TO_POINTER (n));

    ptrs = g_new (GList *, len);

    for (node = list, i = 0; i < len; node = g_list_next (node), i++)
	ptrs[i] = node;

    j = g_random_int_range (0, len);
    list = ptrs[j];
    ptrs[j]->next = NULL;
    ptrs[j] = ptrs[0];

    for (i = 1; i < len; i++)
	{
	    j = g_random_int_range (0, len - i);
	    list->prev = ptrs[i + j];
	    ptrs[i + j]->next = list;
	    list = ptrs[i + j];
	    ptrs[i + j] = ptrs[i];
	}
    list->prev = NULL;

    g_free (ptrs);

    return list;
}

/* counts number of digits in a gint */
guint
gint_count_digits (gint n)
{
    guint count = 0;

    n = ABS (n);
    do {
	count++;
	n /= 10;
    } while (n > 0);

    return count;
}

gchar *
str_replace_char (gchar * str, gchar old, gchar new)
{
    gchar *match;

    g_return_val_if_fail (str != NULL, NULL);

    match = str;
    while ((match = strchr (match, old)))
	*match = new;

    return str;
}

gchar *
str_replace (gchar *str,
	     gchar *new_str)
{
    g_free (str);
    return new_str;
}

gboolean
str_has_prefix_nocase (const gchar *str, const gchar *prefix)
{
    if ((!str) || (!prefix)) return FALSE;
    return (strncasecmp (str, prefix, strlen (prefix)) == 0);
}

gboolean
str_has_suffix_nocase (const gchar *str, const gchar *_suffix)
{
    gchar *suffix_fq;
    gboolean rv;

    if ((!str) || (!_suffix)) return FALSE;

    suffix_fq = g_strconcat (".", _suffix, NULL);
    rv = (strcasecmp (str + strlen (str) - strlen (suffix_fq), suffix_fq) == 0);
    
    g_free (suffix_fq);
    return rv;
}

gboolean
str_has_suffixes_nocase (const gchar * str,
			 gchar * const *suffixes)
{
    gchar *const *suffix;

    g_return_val_if_fail (str != NULL, FALSE);
    g_return_val_if_fail (suffixes != NULL, FALSE);

    for (suffix = suffixes; *suffix; suffix++)
	if (str_has_suffix_nocase (str, *suffix))
	    return TRUE;

    return FALSE;
}

gboolean
bmp_become_daemon (void)
{
  pid_t child_pid;
  int dev_null_fd;

  if (g_chdir ("/") < 0)
    {
      g_message ("Could not change into root directory");
      return FALSE;
    }

  switch ((child_pid = fork ()))
    {
      case -1:
        g_message ("fork failed\n");
        return FALSE;
        break;

      case 0:
        /* silently ignore failures here, if someone
         * doesn't have /dev/null we may as well try
         * to continue anyhow
         */

        dev_null_fd = g_open ("/dev/null", O_RDWR);
        if (dev_null_fd >= 0)
          {
            dup2 (dev_null_fd, 0);
            dup2 (dev_null_fd, 1);
            dup2 (dev_null_fd, 2);
          }

        /* Get a predictable umask */
        umask (022);
        break;

      default:
        exit (0);
        break;
    }

  if (setsid () == -1)
    g_error ("setsid () failed");

  return TRUE;
}

/* NOTE: This is a shallow copy, it only copies the pointers, not the strings themselves */
gchar **
gslist_to_strv (GSList *list)
{
    gchar **strv;
    gint length, n = 0;

    length =
      g_slist_length (list)+1;
    strv =
      g_new (gchar *, length);

    while (list)
	{
	    strv[n] = list->data;
	    n++;
	    list = list->next;
	}

    strv[n] = NULL;

    return strv;
}

/* NOTE: This is a shallow copy, it only copies the pointers, not the strings themselves */
gchar **
glist_to_strv (GList *list)
{
  gchar **strv;
  gint length, n = 0;

  length = g_list_length (list)+1;
  strv = g_new (gchar *, length);

  while (list)
    {
      strv[n] = list->data;
      n++;
      list = list->next;
    }

  strv[n] = NULL;

  return strv;
}

BmpUniqueList *
bmp_unique_list_new (GType type)
{
  BmpUniqueList *ulist = g_new (BmpUniqueList, 1);

  ulist->list = NULL;

  ulist->hash = (type == G_TYPE_STRING) ?
    g_hash_table_new (g_str_hash, g_str_equal) :
    g_hash_table_new (g_direct_hash, g_direct_equal);

  return ulist;
}

void
bmp_unique_list_free (BmpUniqueList *ulist)
{
  g_list_free (ulist->list);
  g_hash_table_destroy (ulist->hash);
  g_free (ulist);
}

void
bmp_unique_list_insert (BmpUniqueList *ulist,
                        gconstpointer  key)
{
  if (!g_hash_table_lookup (ulist->hash, key))
    {
      ulist->list = g_list_append (ulist->list, (gpointer) key);
      g_hash_table_insert (ulist->hash, (gpointer) key, g_list_last (ulist->list));
    }
}

void
bmp_unique_list_remove (BmpUniqueList *ulist,
                        gconstpointer  key)
{
    gpointer llink = g_hash_table_lookup (ulist->hash, key);
    if (llink) {
	g_hash_table_remove (ulist->hash, key);
	ulist->list = g_list_remove_link (ulist->list, llink);
    }
}

GList *
bmp_unique_list_to_glist (BmpUniqueList *ulist)
{
  return ulist->list;
}

gchar *
get_cdda_mrl_from_uri (const gchar *uri)
{
    GURI *g_uri;
    gchar *mrl;
    gchar **fragments;

    g_uri = bmp_uri_new (uri);
    fragments = g_strsplit (g_uri->path,"/",-1);
    mrl = g_strconcat (g_uri->scheme,":/",fragments[1],NULL);
    bmp_uri_delete (g_uri);
    g_strfreev (fragments);
    return mrl;
}


